# -*- coding: utf-8 -*-
import tkinter as tk
import customtkinter
from tkinter import filedialog as fd
from functools import partial
import MistrasDTA
import os
import gc
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
from numpy.lib.recfunctions import join_by

customtkinter.set_appearance_mode("dark")  # Modes: system (default), light, dark
customtkinter.set_default_color_theme("blue")  # Themes: blue (default), dark-blue, green
    
root = customtkinter.CTk()  
root.tk.call('tk', 'scaling', 1.4)
root.geometry('1230x830')     
root.resizable(width=False, height=False)
root.title('DTA waveviewer')  
root.configure(bg='#212325')
plt.style.use(['default', 'dark_background'])
plt.rcParams['axes.facecolor'] = '#212325'
plt.rcParams['figure.facecolor'] = '#333333'
plt.rcParams['figure.edgecolor'] = '#2A2A2A'

counter = tk.IntVar()

#-- Functions --#
def _clear():
    for item in canvas.get_tk_widget().find_all():
       canvas.get_tk_widget().delete(item)

def rmfield( a, *fieldnames_to_remove ):
    return a[ [ name for name in a.dtype.names if name not in fieldnames_to_remove ] ]

def loadfile(label_loadfile, label_loadpath):
    filetypes=(("DTA", ".dta"),)
    filename = fd.askopenfilename(
        title='Open a file',
        initialdir='/',
        filetypes=filetypes)
    if filename is None:
        return
    path = os.path.dirname(filename)
    label_loadfile.config(text="%s" % filename)
    label_loadpath.config(text="%s" % path)
    return 

def extractall(label_numhits):
    if not os.path.exists(label_loadDTA.cget("text")):
        return
    
    global merged
    # Read the binary file
    rec, wfm = MistrasDTA.read_bin(label_loadDTA.cget("text"))
    merged = join_by(['SSSSSSSS.mmmuuun', 'CH'], rec, wfm)
    
    t0, V0 = MistrasDTA.get_waveform_data(merged[0])
    t1, V1 = MistrasDTA.get_waveform_data(merged[1])
    t2, V2 = MistrasDTA.get_waveform_data(merged[2])
    t3, V3 = MistrasDTA.get_waveform_data(merged[3])
    t4, V4 = MistrasDTA.get_waveform_data(merged[4])
    t5, V5 = MistrasDTA.get_waveform_data(merged[5])
    
    Ch0 = merged['CH'][0]
    Ch1 = merged['CH'][1]
    Ch2 = merged['CH'][2]
    Ch3 = merged['CH'][3]
    Ch4 = merged['CH'][4]
    Ch5 = merged['CH'][5]

    Amp0 = merged['AMP'][0]
    Amp1 = merged['AMP'][1]
    Amp2 = merged['AMP'][2]
    Amp3 = merged['AMP'][3]
    Amp4 = merged['AMP'][4]
    Amp5 = merged['AMP'][5]
    
    En0 = merged['ABS-ENERGY'][0]
    En1 = merged['ABS-ENERGY'][1]
    En2 = merged['ABS-ENERGY'][2]
    En3 = merged['ABS-ENERGY'][3]
    En4 = merged['ABS-ENERGY'][4]
    En5 = merged['ABS-ENERGY'][5]
    
    Count0 = merged['COUN'][0]
    Count1 = merged['COUN'][1]
    Count2 = merged['COUN'][2]
    Count3 = merged['COUN'][3]
    Count4 = merged['COUN'][4]
    Count5 = merged['COUN'][5]

    Freq0 = merged['P-FRQ'][0]
    Freq1 = merged['P-FRQ'][1]
    Freq2 = merged['P-FRQ'][2]
    Freq3 = merged['P-FRQ'][3]
    Freq4 = merged['P-FRQ'][4]
    Freq5 = merged['P-FRQ'][5]
    
    Time0 = merged['SSSSSSSS.mmmuuun'][0]
    Time1 = merged['SSSSSSSS.mmmuuun'][1]
    Time2 = merged['SSSSSSSS.mmmuuun'][2]
    Time3 = merged['SSSSSSSS.mmmuuun'][3]
    Time4 = merged['SSSSSSSS.mmmuuun'][4]
    Time5 = merged['SSSSSSSS.mmmuuun'][5]

    Thr = merged['THR'][0]

    plt.cla()
    plt.clf()    
    plt.close()
    fig = plt.figure(figsize=(16,9)  , dpi=72)
    gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
    (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
    for ax in fig.get_axes():
        ax.label_outer()
    
    ax1.plot(t0, V0, 'tab:red')
    ax2.plot(t1, V1, 'tab:orange')
    ax3.plot(t2, V2, 'tab:green')
    ax4.plot(t3, V3, 'tab:blue')
    ax5.plot(t4, V4, 'tab:purple')
    ax6.plot(t5, V5, 'tab:pink')
    
    fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
    fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
        
    fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 3.1, "Viewing hits: 1 - 6", fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    
    fig.text(-0.98, 2.08, "Arrival time: {:.4f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 2.08, "Arrival time: {:.4f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 1.08, "Arrival time: {:.4f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 1.08, "Arrival time: {:.4f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 0.08, "Arrival time: {:.4f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 0.08, "Arrival time: {:.4f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    
    fig.tight_layout(pad=5)
    
    canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
    canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
    
    toolbarFrame = tk.Frame(master=root)
    toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
    toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
    toolbar.config(bg="#212325")
    toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
    toolbar.winfo_children()[-2].config(background="#212325")
    toolbar.update()
    
    numhits = len(merged)
    label_numhits.config(text="%s" % numhits) 

    return

def onClickdec(event=None):
    counter.set(counter.get() - 1)

def onClick(event=None):
    counter.set(counter.get() + 1)
    
def reset(event=None):
    counter.set(0)

def gotobutton(event=None):
    counter.set(int(goto.get()))

def arrow():
    gc.collect() 
    
    if not os.path.exists(label_loadDTA.cget("text")):
        return
    
    num = label_counter.cget("text") * 6
    viewfirst = num + 1
    viewlast = num + 6
        
    t0, V0 = MistrasDTA.get_waveform_data(merged[num])
    t1, V1 = MistrasDTA.get_waveform_data(merged[num + 1])
    t2, V2 = MistrasDTA.get_waveform_data(merged[num + 2])
    t3, V3 = MistrasDTA.get_waveform_data(merged[num + 3])
    t4, V4 = MistrasDTA.get_waveform_data(merged[num + 4])
    t5, V5 = MistrasDTA.get_waveform_data(merged[num + 5])
    
    Ch0 = merged['CH'][num]
    Ch1 = merged['CH'][num + 1]
    Ch2 = merged['CH'][num + 2]
    Ch3 = merged['CH'][num + 3]
    Ch4 = merged['CH'][num + 4]
    Ch5 = merged['CH'][num + 5]
    
    Amp0 = merged['AMP'][num]
    Amp1 = merged['AMP'][num + 1]
    Amp2 = merged['AMP'][num + 2]
    Amp3 = merged['AMP'][num + 3]
    Amp4 = merged['AMP'][num + 4]
    Amp5 = merged['AMP'][num + 5]
    
    En0 = merged['ABS-ENERGY'][num]
    En1 = merged['ABS-ENERGY'][num + 1]
    En2 = merged['ABS-ENERGY'][num + 2]
    En3 = merged['ABS-ENERGY'][num + 3]
    En4 = merged['ABS-ENERGY'][num + 4]
    En5 = merged['ABS-ENERGY'][num + 5]
    
    Count0 = merged['COUN'][num]
    Count1 = merged['COUN'][num + 1]
    Count2 = merged['COUN'][num + 2]
    Count3 = merged['COUN'][num + 3]
    Count4 = merged['COUN'][num + 4]
    Count5 = merged['COUN'][num + 5]

    Freq0 = merged['P-FRQ'][num]
    Freq1 = merged['P-FRQ'][num + 1]
    Freq2 = merged['P-FRQ'][num + 2]
    Freq3 = merged['P-FRQ'][num + 3]
    Freq4 = merged['P-FRQ'][num + 4]
    Freq5 = merged['P-FRQ'][num + 5]
    
    Time0 = merged['SSSSSSSS.mmmuuun'][num]
    Time1 = merged['SSSSSSSS.mmmuuun'][num + 1]
    Time2 = merged['SSSSSSSS.mmmuuun'][num + 2]
    Time3 = merged['SSSSSSSS.mmmuuun'][num + 3]
    Time4 = merged['SSSSSSSS.mmmuuun'][num + 4]
    Time5 = merged['SSSSSSSS.mmmuuun'][num + 5]
    
    Thr = merged['THR'][num]
    
    plt.cla()
    plt.clf()
    plt.close()
    fig = plt.figure(figsize=(16,9)  , dpi=72)
    gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
    (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
    for ax in fig.get_axes():
        ax.label_outer()
    
    ax1.plot(t0, V0, 'tab:red')
    ax2.plot(t1, V1, 'tab:orange')
    ax3.plot(t2, V2, 'tab:green')
    ax4.plot(t3, V3, 'tab:blue')
    ax5.plot(t4, V4, 'tab:purple')
    ax6.plot(t5, V5, 'tab:pink')
    
    fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
    fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
    
    fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 3.1, "Viewing hits: {:d} - {:d}".format(viewfirst, viewlast), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    
    fig.text(-0.98, 2.08, "Arrival time: {:.4f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 2.08, "Arrival time: {:.4f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 1.08, "Arrival time: {:.4f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 1.08, "Arrival time: {:.4f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(-0.98, 0.08, "Arrival time: {:.4f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    fig.text(0.03, 0.08, "Arrival time: {:.4f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
    
    fig.tight_layout(pad=5)
    
    canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
    canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
        
    toolbarFrame = tk.Frame(master=root)
    toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
    toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
    toolbar.config(bg="#212325")
    toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
    toolbar.winfo_children()[-2].config(background="#212325")
    toolbar.update()

    return

#-- GUI ROW BY ROW --# 
#Row 0
label_RunNumber = tk.Label(root, text="Load a Mistras DTA file and view the AE waveforms")
label_RunNumber.grid(row=0, column=0, sticky=tk.W, columnspan=10, pady=(10,10), padx=(10, 0))
label_RunNumber.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')

label_loadDTA = tk.Label(root, text="path") #redirect for another function
label_loadpath = tk.Label(root, text="path") #redirect for another function

label_hits = tk.Label(root, text="Total hits:")
label_hits.grid(row=1, column=1, sticky=tk.E)
label_hits.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')
label_numhits = tk.Label(root, text="    ")
label_numhits.grid(row=1, column=2, sticky=tk.W)
label_numhits.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')

#-- Button --#
loadDTA = partial(loadfile, label_loadDTA, label_loadpath)
extract = partial(extractall, label_numhits)
buttonCal = customtkinter.CTkButton(root, text="Load DTA file", width = 100, command=lambda:[loadDTA(),_clear(),extract()])
buttonCal.grid(row=1, column=0, sticky=tk.W, padx=(15, 5), pady=(2,2))

#Counter
label_counter = tk.Label(root, textvariable=counter)
label_counter.grid(row=1, column=5)
label_counter.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')
counterbutton = customtkinter.CTkButton(root, text=u"\u2190", width = 40, command=lambda:[onClickdec(),_clear(),arrow()])
counterbutton.grid(row=1, column=4, sticky=tk.E, padx=(0, 0))
counterbutton = customtkinter.CTkButton(root, text=u"\u2192", width = 40, command=lambda:[onClick(),_clear(),arrow()])
counterbutton.grid(row=1, column=6, sticky=tk.W)
counterbutton = customtkinter.CTkButton(root, text="Reset", width = 60, command=lambda:[reset(),_clear(),arrow()])
counterbutton.grid(row=1, column=7)

goto = tk.StringVar()
entry_goto = customtkinter.CTkEntry(master=root, textvariable=goto, border_width = 2, width = 50, justify='center')
entry_goto.grid(row=1, column=8, sticky=tk.E, padx=(0, 2), pady=(5,5)) 
entry_goto.insert(0, '0')
counterbutton = customtkinter.CTkButton(root, text="Go to", width = 60, command=lambda:[gotobutton(),_clear(),arrow()])
counterbutton.grid(row=1, column=9, sticky=tk.W)

#blank line
label_Import = tk.Label(root, text="")
label_Import.grid(row=2, column=0, sticky=tk.W, columnspan=10)
label_Import.config(bg="#212325", fg="#FFFFFF")

#Draw plot area
fig = plt.figure(figsize=(16,9)  , dpi=72)
gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
(ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots()
for ax in fig.get_axes():
    ax.label_outer()
fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
fig.tight_layout(pad=5)

canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
    
toolbarFrame = tk.Frame(master=root)
toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
toolbar.config(bg="#212325")
toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
toolbar.winfo_children()[-2].config(background="#212325")
toolbar.update()

label_Note = tk.Label(root, text="Written by R. Farla at P61B    16.12.2022    v1.0")
label_Note.grid(row=5, column=9, columnspan=6, pady=(15,5))
label_Note.config(bg="#212325", fg="#FFFFFF", font='Helvetica 9 bold')

root.mainloop()